// A C++ program that illustrates
// the statistical search algorithm

#include <iostream.h>
#include <iomanip.h>

const int MAX_ELEMS = 10;
const int NOT_FOUND = -1;
const int TRUE = 1;
const int FALSE = 0;

class myArray
{
  public:
    myArray(int fInitVal = 0);
    int& operator[](int nIndex)
      { return m_nArray[nIndex]; }
    void show(const char* pszMsg = "",
              const int nNumElems = MAX_ELEMS,
              const int bOneLine = 1);
    int statSearch(int nKey,
                   int nLast = MAX_ELEMS - 1,
                   int nFirst = 0);
    int getNumCompare()
      { return m_nNumCompare; }

  protected:
    int m_nArray[MAX_ELEMS];
    int m_nNumCompare;
};

myArray::myArray(int fInitVal)
{
  for (int i = 0; i < MAX_ELEMS; i++)
    m_nArray[i] = fInitVal;
}

void myArray::show(const char* pszMsg,
                   const int nNumElems,
                   const int bOneLine)
{
  cout << pszMsg;
  if (bOneLine) {
    for (int i = 0; i < nNumElems; i++)
      cout << m_nArray[i] << ' ';
    cout << endl;
  }
  else {
    for (int i = 0; i < nNumElems; i++)
      cout << m_nArray[i] << endl;
    cout << endl;
  }
}

int myArray::statSearch(int nKey, int nLast, int nFirst)
{
  int nNumElems, m, i, j, k, nCount, nShift;
  int notFound = TRUE;

  if (nFirst > nLast || nFirst >= MAX_ELEMS)
    return NOT_FOUND;

  nNumElems = nLast - nFirst + 1;
  m = nNumElems / 2;
  nCount = m;
  m_nNumCompare = 0;
  // is nNumElems an odd number?
  if ((nNumElems % 2) > 0) {
    m_nNumCompare++;
    // examine the median of an odd-number range
    if (m_nArray[m] == nKey) {
      nKey = m_nArray[m];
      return m;
    }
    nShift = 1;
  }
  else
    nShift = 0;
  k = 2 * m + nShift;
  // search around the median value
  while (notFound && nCount > 0) {
    // search above the median
    j = k - nCount;
    m_nNumCompare++;
    // m_nArray[j] matches search data?
    if (m_nArray[j] == nKey) {
       // save matching index in k
       k = j;
       notFound = FALSE;
    }

    // still not found a match
    if (notFound) {
      // search below the median and also
      // decrement nCount
      i = nCount-- - 1;
      m_nNumCompare++;
      // m_nArray[i] matches search data?
      if (m_nArray[i] == nKey) {
        // save matching index in k
        k = i;
        notFound = FALSE;
      }
    }
  }

  return (notFound) ? NOT_FOUND : k;
}

main()
{
  int j;
  int nArr[MAX_ELEMS] = { 23, 45, 89, 74, 51,
                          18, 87, 63, 36, 38 };
  myArray Array;

  for (int i = 0; i < MAX_ELEMS; i++)
    Array[i] = nArr[i];

  Array.show("Array is: ");

  for (i = 0; i < MAX_ELEMS; i += 3) {
    cout << "Searching for " << nArr[i];
    j = Array.statSearch(nArr[i]);
    if (j != NOT_FOUND)
      cout << " found match at index "
           << j << " (" << Array.getNumCompare()
           << " comparisons)" << endl;
    else
      cout << " found no match\n";
  }
  return 0;
}

