// A C++ program that illustrates
// the binary search algorithm

#include <iostream.h>
#include <iomanip.h>

const int MAX_ELEMS = 10;
const int NOT_FOUND = -1;
const int SORTED = 1;
const int UNSORTED = 0;

class myArray
{
  public:
    myArray(int fInitVal = 0);
    int& operator[](int nIndex)
      { return m_nArray[nIndex]; }
    void show(const char* pszMsg = "",
              const int nNumElems = MAX_ELEMS,
              const int bOneLine = 1);
    void QuickSort(int nNumElems = MAX_ELEMS);
    void clearSortFlag()
      { m_bSorted = UNSORTED; }
    int binarySearch(int nKey,
                     int nLast = MAX_ELEMS - 1,
                     int nFirst = 0);

  protected:
    int m_nArray[MAX_ELEMS];
    int m_bSorted;

    void QSort(int nFirst, int nLast);
};

myArray::myArray(int fInitVal)
{
  for (int i = 0; i < MAX_ELEMS; i++)
    m_nArray[i] = fInitVal;
  m_bSorted = UNSORTED;
}

void myArray::show(const char* pszMsg,
                   const int nNumElems,
                   const int bOneLine)
{
  cout << pszMsg << endl;
  if (bOneLine) {
    for (int i = 0; i < nNumElems; i++)
      cout << m_nArray[i] << ' ';
    cout << endl;
  }
  else {
    for (int i = 0; i < nNumElems; i++)
      cout << m_nArray[i] << endl;
    cout << endl;
  }
}

void myArray::QuickSort(int nNumElems)
{
  if (!m_bSorted && nNumElems > 1) {
    QSort(0, nNumElems - 1);
    m_bSorted = SORTED;
  }
}

void myArray::QSort(int nFirst, int nLast)
{
  int i = nFirst, j = nLast;
  int nMedian;

  // obtain the median element
  nMedian = m_nArray[(nFirst + nLast) / 2];
  do {
    while (m_nArray[i] < nMedian)
      i++;
    while (nMedian < m_nArray[j])
      j--;
    if (i <= j) {
      // swap elements
      int nSwap = m_nArray[i];
      m_nArray[i++] = m_nArray[j];
      m_nArray[j--] = nSwap;
    }
  } while (i <= j);

  if (nFirst < j)
    QSort(nFirst, j);

  if (i < nLast)
    QSort(i, nLast);
}

int myArray::binarySearch(int nKey, int nLast, int nFirst)
{
  int nMedian;

  if (!m_bSorted)
    return NOT_FOUND;

  do {
    nMedian = (nFirst + nLast) / 2;
    if (nKey < m_nArray[nMedian])
      nLast = nMedian - 1;
    else
      nFirst = nMedian + 1;
  } while (!(nKey == m_nArray[nMedian] || nFirst > nLast));

  return (nKey == m_nArray[nMedian]) ? nMedian : NOT_FOUND;

}

main()
{
  int j;
  int nArr[MAX_ELEMS] = { 23, 45, 89, 74, 51,
                          18, 87, 63, 36, 38 };
  myArray Array;

  for (int i = 0; i < MAX_ELEMS; i++)
    Array[i] = nArr[i];

  Array.QuickSort();
  Array.show("Sorted array is:");
  for (i = 0; i < MAX_ELEMS; i += 3) {
    cout << "Searching for " << nArr[i];
    j = Array.binarySearch(nArr[i]);
    if (j != NOT_FOUND)
      cout << " found match at index " << j << endl;
    else
      cout << " found no match\n";
  }

  return 0;
}

