// A C++ program that illustrates
// the heuristic search algorithm

#include <iostream.h>
#include <iomanip.h>

const int MAX_ELEMS = 10;
const int NOT_FOUND = -1;

class myArray
{
  public:
    myArray(int fInitVal = 0);
    int& operator[](int nIndex)
      { return m_nArray[nIndex]; }
    void show(const char* pszMsg = "",
              const int nNumElems = MAX_ELEMS,
              const int bOneLine = 1);
    int heuristicSearch(int nKey,
                        int nLast = MAX_ELEMS - 1,
                        int nFirst = 0);

  protected:
    int m_nArray[MAX_ELEMS];
};

myArray::myArray(int fInitVal)
{
  for (int i = 0; i < MAX_ELEMS; i++)
    m_nArray[i] = fInitVal;
}

void myArray::show(const char* pszMsg,
                   const int nNumElems,
                   const int bOneLine)
{
  cout << pszMsg;
  if (bOneLine) {
    for (int i = 0; i < nNumElems; i++)
      cout << m_nArray[i] << ' ';
    cout << endl;
  }
  else {
    for (int i = 0; i < nNumElems; i++)
      cout << m_nArray[i] << endl;
    cout << endl;
  }
}

int myArray::heuristicSearch(int nKey, int nLast, int nFirst)
{
  for (int i = nFirst; i <= nLast; i++)
    if (m_nArray[i] == nKey)
      break;

  if (i <= nLast) {
    if (i > 0) {
       // swap elements at indices i and i-1
      int nSwap = m_nArray[i];
      m_nArray[i] = m_nArray[i-1];
      m_nArray[--i] = nSwap;
    }
    return i;
  }
  else
    return NOT_FOUND;
}

main()
{
  int j;
  int nLast = MAX_ELEMS - 1;
  int nArr[MAX_ELEMS] = { 23, 45, 89, 74, 51,
                          18, 87, 63, 36, 38 };
  myArray Array;

  for (int i = 0; i < MAX_ELEMS; i++)
    Array[i] = nArr[i];

  Array.show("Initial heuristic array is: ");

  for (i = 0; i < MAX_ELEMS / 2; i++) {
    cout << "Searching for " << nArr[nLast];
    j = Array.heuristicSearch(nArr[nLast]);
    if (j != NOT_FOUND)
      cout << " found match at index " << j << endl;
    else
      cout << " found no match\n";
    Array.show("Heuristic array is: ");
  }
  return 0;
}

