// A C++ program that illustrates the break statement

#include <iostream.h>

const double DEF_TOLR = 1.0e-08;
const int MAX_TERMS = 100;
const double EPSILON = 1.0e-50;
const double BAD_RESULT = -1.0e+30;

class SumPoly
{
  public:
    SumPoly(const double fTolerance = DEF_TOLR,
            const int nMaxTerms = MAX_TERMS)
      { setParams(fTolerance, nMaxTerms); }
    void setParams(const double fTolerance = DEF_TOLR,
                   const int nMaxTerms = MAX_TERMS);
    double polynomial(double x);

  protected:
    double m_fTolerance;
    double m_nMaxTerms;

    double abs(double x)
      {  return (x >= 0) ? x : -x; }
};

void SumPoly::setParams(const double fTolerance,
                        const int nMaxTerms)
{
  m_fTolerance = fTolerance;
  m_nMaxTerms = nMaxTerms;
}

double SumPoly::polynomial(double x)
{
  double fSum = 1;
  double fPow = 1;
  double fTerm;
  double fChs = 1;
  int nTerm = 1;

  if (abs(x) < EPSILON)
    return BAD_RESULT;

  do {
    // update power of x
    fPow *= x;
    // calculate term
    fTerm = fChs / fPow;
    // update summation
    fSum += fTerm;
    // is calculated term too small?
    if (abs(fTerm) < m_fTolerance)
      break;
    // toggle sign
    fChs *= -1.;
    // update counter for the number of terms
    nTerm++;
  } while (nTerm < m_nMaxTerms);

  return fSum;
}

main()
{
  const double LO = 1.0;
  const double HI = 5.0;
  SumPoly Poly;
  double fX = LO;

  while (fX <= HI) {
    cout << "P(" << fX << ") = "
         << Poly.polynomial(fX) << "\n";
    fX++;
  }

  return 0;
}

