// A C++ program that illustrates
// declaring and using virtual functions

#include <iostream.h>
#include <iomanip.h>
#include <math.h>

const double PI = 4 * atan(1);

class BasicStat1
{
  public:
    BasicStat1()
      { clearSums(); }
    void clearSums();
    void addRandData(int nNumObs);
    void getStat(int& nSum, double& fMean, double& fSdev);
    virtual double random();

  protected:
    int m_nSum;
    double m_fSumX;
    double m_fSumXX;
    double m_fSeed;

    double sqr(double x)
      { return x * x; }
    double frac(double x)
      { return x - int(x); }
};

class BasicStat2 : public BasicStat1
{
  public:
    BasicStat2()
      { clearSums(); }
    virtual double random();

  protected:
    double cube(double x)
      { return x * x * x; }
};

class BasicStat3 : public BasicStat2
{
  public:
    BasicStat3()
      { clearSums(); }
    virtual double random();

  protected:
    double fourth(double x)
      { return sqr(x) * sqr(x); }
};

void BasicStat1::clearSums()
{
  m_nSum = 0;
  m_fSumX = 0;
  m_fSumXX = 0;
  m_fSeed = 0.123456789;
}

void BasicStat1::addRandData(int nNumObs)
{
  for (int i = 0; i < nNumObs; i++) {
    double x = random();
    m_fSumX += x;
    m_fSumXX += sqr(x);
  }
  if (nNumObs > 0)
    m_nSum += nNumObs;
}

void BasicStat1::getStat(int& nSum, double& fMean, double& fSdev)
{
  nSum = m_nSum;
  if (m_nSum > 0) {
    fMean = m_fSumX / m_nSum;
    fSdev = sqrt((m_fSumXX - sqr(m_fSumX) / m_nSum)/(m_nSum - 1));
  }
}

double BasicStat1::random()
{
  m_fSeed = frac(sqr(PI + m_fSeed));
  return m_fSeed;
}

double BasicStat2::random()
{
  m_fSeed = frac(cube(PI + m_fSeed));
  return m_fSeed;
}

double BasicStat3::random()
{
  m_fSeed = frac(fourth(PI + m_fSeed));
  return m_fSeed;
}

main()
{
  const int MAX = 1000;

  int nSum;
  double fMean;
  double fSdev;
  BasicStat1 BasStat1;
  BasicStat2 BasStat2;
  BasicStat3 BasStat3;

  cout << "Testing the random number generator # 1\n";
  BasStat1.addRandData(MAX);
  BasStat1.getStat(nSum, fMean, fSdev);
  cout << "Number of observations = " << nSum << endl
       << "Mean value             = " << fMean << endl
       << "Standard deviation     = " << fSdev << endl
       << endl;

  cout << "Testing the random number generator # 2\n";
  BasStat2.addRandData(MAX);
  BasStat2.getStat(nSum, fMean, fSdev);
  cout << "Number of observations = " << nSum << endl
       << "Mean value             = " << fMean << endl
       << "Standard deviation     = " << fSdev << endl
       << endl;

  cout << "Testing the random number generator # 3\n";
  BasStat3.addRandData(MAX);
  BasStat3.getStat(nSum, fMean, fSdev);
  cout << "Number of observations = " << nSum << endl
       << "Mean value             = " << fMean << endl
       << "Standard deviation     = " << fSdev << endl;

  return 0;
}

