// A C++ program that illustrates
// declaring and using friend operators

#include <iostream.h>
#include <string.h>

const int MIN_SIZE = 30;

class myString
{
  public:
    myString(int nSize = MIN_SIZE)
      { m_pszString = new char[m_nSize = nSize]; }
    myString(unsigned nSize)
      { m_pszString = new char[m_nSize = int(nSize)]; }
    myString(myString& aString);
    myString(const char* pszString);
    myString(const char cChar);
    ~myString()
      { delete [] m_pszString; }
    int getLen()
      { return strlen(m_pszString); }
    int getMaxLen()
      { return m_nSize; }
    myString& operator =(myString& aString);
    myString& operator =(const char* pszString);
    myString& operator =(const char cChar);
    myString& operator +=(myString& aString);
    myString& operator +=(const char* pszString);
    myString& operator +=(const char cChar);
    friend myString operator +(myString& aString1,
                               myString& aString2);
    friend myString operator +(myString& aString1,
                               const char* pszString2);
    friend myString operator +(const char* pszString1,
                               myString& aString2);
    friend myString operator +(myString& aString,
                               const char cChar);
    friend myString operator +(const char cChar,
                               myString& aString);
    void show(const char* pszMsg = "");

  protected:
    char* m_pszString;
    int m_nSize;
};

myString::myString(myString& aString)
{
  m_pszString = new char[m_nSize = aString.m_nSize];
  strcpy(m_pszString, aString.m_pszString);
}

myString::myString(const char* pszString)
{
  m_pszString = new char[m_nSize = strlen(pszString) + 1];
  strcpy(m_pszString, pszString);
}

myString::myString(const char cChar)
{
  m_pszString = new char[m_nSize = MIN_SIZE];
  m_pszString[0] = cChar;
  m_pszString[1] = '\0';
}

myString& myString::operator =(myString& aString)
{
  // source string small enough to copy?
  if (strlen(aString.m_pszString) < unsigned(m_nSize))
    strcpy(m_pszString, aString.m_pszString);
  else
    // copy first m_nSize - 1 characters
    strncpy(m_pszString, aString.m_pszString, m_nSize - 1);
  return *this;
}

myString& myString::operator =(const char* pszString)
{
  // source string small enough to copy?
  if (strlen(pszString) < unsigned(m_nSize))
    strcpy(m_pszString, pszString);
  else
    // copy first m_nSize - 1characters
    strncpy(m_pszString, pszString, m_nSize - 1);
  return *this;
}

myString& myString::operator =(const char cChar)
{
  if (m_nSize > 1) {
    m_pszString[0] = cChar;
    m_pszString[1] = '\0';
  }
  return *this;
}

myString operator +(myString& aString1, myString& aString2)
{
  myString Result(strlen(aString1.m_pszString) +
                  strlen(aString2.m_pszString) + 1);
  strcpy(Result.m_pszString, aString1.m_pszString);
  strcat(Result.m_pszString, aString2.m_pszString);
  return Result;
}

myString operator +(myString& aString1, const char* pszString2)
{
  myString Result(strlen(aString1.m_pszString) +
                  strlen(pszString2) + 1);
  strcpy(Result.m_pszString, aString1.m_pszString);
  strcat(Result.m_pszString, pszString2);
  return Result;
}

myString operator +(const char* pszString1, myString& aString2)
{
  myString Result(strlen(aString2.m_pszString) +
                  strlen(pszString1) + 1);
  strcpy(Result.m_pszString, pszString1);
  strcat(Result.m_pszString, aString2.m_pszString);
  return Result;
}

myString operator +(myString& aString, const char cChar)
{
  int len = strlen(aString.m_pszString);
  myString Result(len + 2);

  strcpy(Result.m_pszString, aString.m_pszString);
  Result.m_pszString[len] = cChar;
  Result.m_pszString[len+2] = '\0';
  return Result;
}

myString operator +(const char cChar, myString& aString)
{
  int len = strlen(aString.m_pszString);
  myString Result(len + 2);

  strcpy(Result.m_pszString, aString.m_pszString);
  Result.m_pszString[0] = cChar;
  Result.m_pszString[1] = '\0';
  strcat(Result.m_pszString, aString.m_pszString);
  return Result;
}

myString& myString::operator +=(myString& aString)
{
  // source string small enough to copy?
  if (strlen(aString.m_pszString) + strlen(m_pszString) <=
      unsigned(m_nSize))
    strcat(m_pszString, aString.m_pszString);
  else
    // copy first m_nSize characters
    strncat(m_pszString, aString.m_pszString,
            m_nSize - strlen(m_pszString) - 1);
  return *this;
}

myString& myString::operator +=(const char* pszString)
{
  // source string small enough to copy?
  if (strlen(pszString) + strlen(m_pszString) <=
      unsigned(m_nSize))
    strcat(m_pszString, pszString);
  else
    // copy first m_nSize characters
    strncat(m_pszString, pszString,
            m_nSize - strlen(m_pszString) - 1);
  return *this;
}

myString& myString::operator +=(const char cChar)
{
  int nWorkSize = strlen(m_pszString);

  // source string small enough to copy?
  if (nWorkSize + 1 <= m_nSize) {
    m_pszString[nWorkSize] = cChar;
    m_pszString[nWorkSize + 1] = '\0';
  }
  return *this;
}

void myString::show(const char* pszMsg)
{
  cout << pszMsg << m_pszString << "\n";
}

main()
{
  const int MAX_CHARS = 81;
  myString aString1(MAX_CHARS);
  myString aString2(MAX_CHARS);
  myString aString3(MAX_CHARS);

  char cString1[] = "ABCDEFG";
  char cString2[] = "567890";
  char cChar = '!';

  aString1 = "||";
  aString2 = "..";

  aString3 = cChar    +     // character
             aString1 +     // string object
             cString1 +     // ASCIIZ string
             aString2 +     // string object
             cString2;      // ASCIIZ string

  aString3.show("String object 3 is ");

  return 0;
}

