// C++ program that illustrates untagged enumerated types

#include <iostream.h>
#include <iomanip.h>

enum Logical { FALSE, TRUE };
// declare untagged enumerated types
enum  { SecondsPerMinute = 60,
        MinutesPerHour = 60,
        HoursPerDay = 24 };

class TimeCalc
{
  public:
    // constructor
    TimeCalc(int nHour = 0, int nMinute = 0, int nSecond = 0)
      { setTime(nHour, nMinute, nSecond); }

    Logical setTime(int nHour = 0,
                    int nMinute = 0,
                    int nSecond = 0);
    long getTime(int& nHour, int& nMinute, int& nSecond);
    long getTimeDiff(TimeCalc& TimeObj);
    void show(const char* pszMsg = "", Logical bEmitCR = TRUE);
    friend ostream& operator<<(ostream& os,
                               const TimeCalc& TimeObj);

  protected:
    int m_nSecond;
    int m_nMinute;
    int m_nHour;

    Logical validate(int nValue, int nMaxValue);
};

Logical TimeCalc::validate(int nValue, int nMaxValue)
{
  return (nValue >= 0 && nValue <= nMaxValue) ? TRUE : FALSE;
}

Logical TimeCalc::setTime(int nHour, int nMinute, int nSecond)
{
  if (validate(nHour, HoursPerDay - 1)        &&
      validate(nMinute, MinutesPerHour - 1)   &&
      validate(nSecond, SecondsPerMinute - 1))  {
      m_nHour = nHour;
      m_nMinute = nMinute;
      m_nSecond = nSecond;
      return TRUE;
  }
  else
    return FALSE;
}

long TimeCalc::getTime(int& nHour, int& nMinute, int& nSecond)
{
  long lResult;

  nHour = m_nHour;
  nMinute = m_nMinute;
  nSecond = m_nSecond;

  // calculate the total number of seconds
  // between midnight and the set time
  lResult = (long(m_nHour) * * MinutesPerHour + m_nMinute) *
            SecondsPerMinute + m_nSecond;

  return lResult;
}

long TimeCalc::getTimeDiff(TimeCalc& TimeObj)
{
  long lResult = TimeObj.m_nSecond - m_nSecond;

  lResult += (TimeObj.m_nMinute - m_nMinute) * SecondsPerMinute;
  lResult += (TimeObj.m_nHour - m_nHour) *
              MinutesPerHour * SecondsPerMinute;
  return lResult;
}

void TimeCalc::show(const char* pszMsg, Logical bEmitCR)
{
  cout << pszMsg;
  cout.fill('0');
  cout.flags(ios::right);
  cout.width(2);
  cout << m_nHour << ":";
  cout.width(2);
  cout << m_nMinute << ":";
  cout.width(2);
  cout << m_nSecond;
  if (bEmitCR)
    cout << endl;
}

ostream& operator<<(ostream& os, const TimeCalc& TimeObj)
{
  os.fill('0');
  os.flags(ios::right);
  os.width(2);
  os << TimeObj.m_nHour << ":";
  os.width(2);
  os << TimeObj.m_nMinute << ":";
  os.width(2);
  os << TimeObj.m_nSecond;
  return os;
}

main()
{
  const TimeCalc Midnight;
  int nHour, nMinute, nSecond;
  TimeCalc StartTime, EndTime;

  StartTime.setTime(12, 0, 0); // set start time to noon
  StartTime.getTime(nHour, nMinute, nSecond);
  // set EndTime to end after 6 hours, 30 minutes and 10 seconds
  EndTime.setTime(nHour + 6, nMinute + 30, nSecond + 10);
  StartTime.show("Start time is ");
  EndTime.show("End time is ");
  cout << "There are "
       << StartTime.getTime(nHour, nMinute, nSecond)
       << " seconds between "
       << StartTime << " and " << Midnight << endl;

  cout << "There are "
       << EndTime.getTime(nHour, nMinute, nSecond)
       << " seconds between "
       << EndTime << " and " << Midnight << endl;

  cout << "There are "
       << StartTime.getTimeDiff(EndTime)
       << " seconds between "
       << EndTime << " and " << StartTime << endl;

  return 0;
}

