// A C++ program that illustrates enumerated types
// used as function parameters

#include <iostream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

const double PI = 4 * atan(1);
const double RAD2DEG = 180.0 / PI;    
const double EPSILON = 1.0e-9;

inline double sqr(double x)
{
  return x * x;
}

enum Direction { Left, Right, Up, Down }; 
enum Logical { FALSE, TRUE };

class Rover
{ 
  public:
    Rover(int nX = 0, int nY = 0);
    void Move(int nDisplacement, Direction eDir);
    void Show(const char* pszMsg = "", Logical bEmitCR = TRUE);
    double getDistanceTraveled()
      {
         return sqrt(sqr(m_nX - m_nX0) +
                     sqr(m_nY - m_nY0));
      }                    
    double getAngle();

  protected:
    int m_nX;
    int m_nY;    
    int m_nX0;
    int m_nY0;
};

Rover::Rover(int nX, int nY)
{
  m_nX = nX;
  m_nY = nY;
  m_nX0 = nX;
  m_nY0 = nY;
}

void Rover::Move(int nDisplacement, Direction eDir)
{
  switch (eDir) {
    case Left:
      m_nX -= nDisplacement;
      break;
      
    case Right:
      m_nX += nDisplacement;
      break;
      
    case Down:
      m_nY -= nDisplacement;
      break;
      
    case Up:
      m_nY += nDisplacement;
      break;
  }
}

void Rover::Show(const char* pszMsg, Logical bEmitCR)
{
  cout << pszMsg;
  cout << "(" << m_nX << ", " << m_nY << ")";
  if (bEmitCR)
    cout << endl;
}

double Rover::getAngle()
{
  double fX = double(m_nX - m_nX0);
  double fY = double(m_nY - m_nY0);
  
  return (fabs(fX) > EPSILON) ? 
          atan(fY / fX) * RAD2DEG : 90;
}


main()
{
  const int MAX = 5;    
  const int MIN_DIST = 1;
  const int MAX_DIST = 100;
  Rover RollingStone;
  int nDist;
  Direction eDir;

  // reseed random-number generator                               
  srand((unsigned)time(NULL));    

  for (int i = 0; i < MAX; i++) {
    nDist = rand() % MAX_DIST + MIN_DIST;
    eDir = Direction(rand() % 4);
    RollingStone.Move(nDist, eDir);
    RollingStone.Show("Object is at ");
    cout << "Distance from starting point = "
         << RollingStone.getDistanceTraveled() << endl
         << "Angle = "
         << RollingStone.getAngle()
         << " degrees\n";
  }
  
  return 0;
}

