// A C++ program that illustrates
// random access files

#include <fstream.h>
#include <iomanip.h>

enum Logical { FALSE, TRUE };

const int BAD_VALUE = -32678;

class myArray
{
  public:
    myArray(const char* pszFilename, int nSize, int nInitVal = 0);
    ~myArray()
      { m_f.close(); }
    void store(int nVal, int nIndex);
    int recall(int nIndex);
    void show(const char* pszMsg,
              const int nNumElems,
              const int bOneLine = TRUE);
    void CombSort(int nNumElems);

  protected:
    fstream m_f;
    int m_nSize;
};

myArray::myArray(const char* pszFilename, int nSize, int nInitVal)
{
  m_f.open(pszFilename, ios::in | ios::out | ios::binary);
  m_nSize = nSize;
  for (int i = 0; i < m_nSize; i++)
    m_f.write((const char*)&nInitVal, sizeof(nInitVal));
}

void myArray::show(const char* pszMsg,
                   const int nNumElems,
                   const int bOneLine)
{
  int nVal;

  // set stream pointer to beginning of file
  m_f.seekg(0);
  cout << pszMsg << endl;
  if (bOneLine) {
    for (int i = 0; i < nNumElems; i++) {
      m_f.read((char*)&nVal, sizeof(nVal));
      cout << nVal << ' ';
    }
    cout << endl;
  }
  else {
    for (int i = 0; i < nNumElems; i++) {
      m_f.read((char*)&nVal, sizeof(nVal));
      cout << nVal << endl;
    }
    cout << endl;
  }
}

void myArray::store(int nVal, int nIndex)
{
  if (nIndex < m_nSize) {
    m_f.seekg(nIndex * sizeof(int));
    m_f.write((const char*)&nVal, sizeof(nVal));
  }
}

int myArray::recall(int nIndex)
{
  int nVal = BAD_VALUE;

  if (nIndex < m_nSize) {
    m_f.seekg(nIndex * sizeof(int));
    m_f.read((char*)&nVal, sizeof(nVal));
  }
  return nVal;
}


void myArray::CombSort(int nNumElems)
{
  int nOffset = nNumElems;
  int bSorted;
  int elemI, elemJ;

  if (nNumElems < 2)
    return;

  do {
    nOffset = (nOffset * 8) / 11;
    nOffset = (nOffset < 1) ? 1 : nOffset;
    bSorted = TRUE; // set sorted flag
    // compare elements
    for (int i = 0, j = nOffset;
         i < (nNumElems - nOffset);
         i++, j++) {
      m_f.seekg(i * sizeof(int));
      m_f.read((char*)&elemI, sizeof(elemI));
      m_f.seekg(j * sizeof(int));
      m_f.read((char*)&elemJ, sizeof(elemJ));
      if (elemI > elemJ) {
        // swap elements
        m_f.seekg(i * sizeof(int));
        m_f.write((const char*)&elemJ, sizeof(elemJ));
        m_f.seekg(j * sizeof(int));
        m_f.write((const char*)&elemI, sizeof(elemI));
        bSorted = FALSE; // clear sorted flag
      }
    }
  } while (!bSorted || nOffset != TRUE);
}

main()
{
  const int MAX_ELEMS = 10;
  int nArr[MAX_ELEMS] = { 23, 45, 89, 74, 51,
                          18, 87, 63, 36, 38 };
  myArray Array("RAND.DAT", MAX_ELEMS);

  for (int i = 0; i < MAX_ELEMS; i++)
    Array.store(nArr[i], i);

  Array.show("Unsorted array is:", MAX_ELEMS);
  Array.CombSort(MAX_ELEMS);
  Array.show("Sorted array is:", MAX_ELEMS);

  return 0;
}

