// A C++ program that illustrates nested loops

#include <iostream.h>
#include <math.h>

const double EPSILON = 1.0e-50;
const double BAD_RESULT = -1.0e+30;
const double INIT_SEED = 113;
const int MAX_WIDTH = 40;
const double PI = 4 * atan(1);

class Histogram
{
  public:
    Histogram(double fSeed = INIT_SEED)
      { m_fSeed = fSeed; }
    void setParams(int nNumHisto, double fMax);
    void show();

  protected:
    double m_fSeed;
    double m_fMax;
    int m_nNumHisto;

    double random();
    double frac(double x)
      { return x - long(x); }
    double cube(double x)
      { return x * x * x; }
};

void Histogram::setParams(int nNumHisto, double fMax)
{
  m_nNumHisto = (nNumHisto > 2) ? nNumHisto : 3;
  m_fMax = (fMax > 0) ? fMax : 10;
}

void Histogram::show()
{
  double fX;
  int nVal;

  for (int i = 0; i < m_nNumHisto; i++) {
    fX = m_fMax * random() + 1;
    nVal = int(fX * m_fMax) / MAX_WIDTH;
    // nested for loop
    for (int j = 0; j < nVal; j++)
      cout << "*";
    // nested while loop
    while (nVal++ < MAX_WIDTH)
      cout << ' ';
    if (fX < 10.0) {
      cout << ' ';
      cout.width(4);
      cout.precision(3);
    }
    else {
      cout.width(5);
      cout.precision(4);
    }
    cout << fX << "\n";
  }
}

double Histogram::random()
{
  m_fSeed = frac(cube(m_fSeed + PI));
  return m_fSeed;
}

main()
{
  Histogram Histo;
  int nNumHisto = 10;
  double fMax = 40.0;

  Histo.setParams(nNumHisto, fMax);
  cout << "Sample histogram\n";
  Histo.show();

  return 0;
}

