// A C++ program that illustrates
// the combsort algorithm for matrices

#include <iostream.h>
#include <iomanip.h>

const int MAX_ROWS = 5;
const int MAX_COLS = 3;

class myMatrix
{
  public:
    myMatrix(int fInitVal = 0);
    int& operator()(int nRow, int nCol)
      { return m_nMatrix[nRow][nCol]; }
    void show(const char* pszMsg = "",
              const int nNumRows = MAX_ROWS,
              const int nNumCols = MAX_COLS);
    void CombSort(int nSortColIndex = 0,
                  int nNumRows = MAX_ROWS,
                  int nNumCols = MAX_COLS);

  protected:
    int m_nMatrix[MAX_ROWS][MAX_COLS];

    void swapRows(int nRow1, int nRow2,
                  int nNumCols = MAX_COLS);
};

myMatrix::myMatrix(int fInitVal)
{
  for (int i = 0; i < MAX_ROWS; i++)
    for (int j = 0; j < MAX_COLS; j++)
      m_nMatrix[i][j] = fInitVal;
}

void myMatrix::show(const char* pszMsg,
                    const int nNumRows,
                    const int nNumCols)
{
  cout << pszMsg << endl;
  for (int i = 0; i < nNumRows; i++) {
    for (int j = 0; j < nNumCols; j++)
      cout << m_nMatrix[i][j] << ' ';
    cout << endl;
  }
  cout << endl;
}

void myMatrix::CombSort(int nSortColIndex,
                        int nNumRows,
                        int nNumCols)
{
  int nOffset = nNumRows;
  int bSorted, i, j, ii, jj, nSwapBuffer;
  int nIndex[MAX_ROWS];
  int nRevIndex[MAX_ROWS];

  if (nNumRows < 2)
    return;

  // initialize array of indices
  for (i = 0; i < nNumRows; i++)
    nIndex[i] = i;

  do {
    nOffset = (nOffset * 8) / 11;
    nOffset = (nOffset < 1) ? 1 : nOffset;
    bSorted = 1; // set sorted flag
    // compare elements
    for (i = 0, j = nOffset;
         i < (nNumRows - nOffset);
         i++, j++) {
      ii = nIndex[i];
      jj = nIndex[j];
      if (m_nMatrix[ii][nSortColIndex] >
          m_nMatrix[jj][nSortColIndex]) {
        // swap indices
        nIndex[i] = jj;
        nIndex[j] = ii;
        bSorted = 0; // clear sorted flag
      }
    }
  } while (!bSorted || nOffset != 1);

  // set up the reverse indices
  for (i = 0; i < nNumRows; i++)
    nRevIndex[nIndex[i]] = i;

  // swap rows
  for (i = 0; i < (nNumRows - 1); i++) {
    // swap row number i?
    if (nIndex[i] != i) {
      swapRows(nIndex[i], i);
      ii = nIndex[nRevIndex[i]];
      nIndex[nRevIndex[i]] = nIndex[i];
      nSwapBuffer = nRevIndex[i];
      nRevIndex[i] = nRevIndex[ii];
      nRevIndex[ii] = nSwapBuffer;
    }
  }
}

void myMatrix::swapRows(int nRow1, int nRow2, int nNumCols)
{
  int nSwapBuffer;

  for (int j = 0; j < nNumCols; j++) {
    nSwapBuffer = m_nMatrix[nRow1][j];
    m_nMatrix[nRow1][j] = m_nMatrix[nRow2][j];
    m_nMatrix[nRow2][j] = nSwapBuffer;
  }
}

main()
{
  int nMat[MAX_ROWS][MAX_COLS] =
      { 53, 45, 89, // row 0
        74, 51, 18, // row 1
        87, 63, 36, // row 2
        38, 15, 33, // row 3
        67, 93, 83  // row 4
       };
  myMatrix Matrix;

  for (int i = 0; i < MAX_ROWS; i++)
    for (int j = 0; j < MAX_COLS; j++)
      Matrix(i, j) = nMat[i][j];

  Matrix.show("Unsorted matrix is:");
  Matrix.CombSort();
  Matrix.show("Sorted matrix is:");

  return 0;
}

