// A C++ program that illustrates pointers to structures
// used as function parameters

#include <iostream.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

// declare the structure for basic statistical data
struct BasicStat {
  int m_nNumData;
  double m_fSumOfX;
  double m_fSumOfXSqr;
  double m_fMean;
  double m_fSdev;
};

// declare function prototypes
void initBasicStat(BasicStat* pStatData);
void addData(double* pfArray, int nNumElems,
             BasicStat* pStatData);
void calcBasicStat(BasicStat* pStatData);


main()
{
  const int MAX1 = 100;
  const int MAX2 = 2 * MAX1;
  const int MIN_NUM = 1;
  const int MAX_NUM = 1000;
  double fData1[MAX1];
  double fData2[MAX2];
  // declare a structure
  BasicStat StatData1, StatData2;
  // declare pointers to structures StatData1 and StatData2
  BasicStat* pStatData1 = &StatData1;
  BasicStat* pStatData2 = &StatData2;

  // reseed random-number generator
  srand((unsigned)time(NULL));

  // assign data to array fData1
  for (int i = 0; i < MAX1; i++)
    fData1[i] = double(rand() % MAX_NUM + MIN_NUM);

  // assign data to array fData2
  for (i = 0; i < MAX2; i++)
    fData2[i] = double(rand() % MAX_NUM + MIN_NUM);


  // initialize structures
  initBasicStat(pStatData1);
  initBasicStat(pStatData2);

  // update statistical summations
  addData(fData1, MAX1, pStatData1);
  addData(fData2, MAX2, pStatData2);

  // calculate basic stats for the two structures
  calcBasicStat(pStatData1);
  calcBasicStat(pStatData2);

  cout << "Number of observations in set # 1 = "
       << StatData1.m_nNumData << "\n"
       << "Mean = " << StatData1.m_fMean << "\n"
       << "Standard deviation = "
       << StatData1.m_fSdev << "\n\n";

  cout << "Number of observations in set # 2 = "
       << StatData2.m_nNumData << "\n"
       << "Mean = " << StatData2.m_fMean << "\n"
       << "Standard deviation = "
       << StatData2.m_fSdev;

  return 0;
}

void initBasicStat(BasicStat* pStatData)
{
  // initialize some of the data members
  pStatData->m_nNumData = 0;
  pStatData->m_fSumOfX = 0;
  pStatData->m_fSumOfXSqr = 0;
}

void addData(double* pfArray, int nNumElems,
             BasicStat* pStatData)
{
  pStatData->m_nNumData += nNumElems;
  // obtain statistical summations
  for (int i = 0; i < nNumElems; i++) {
    pStatData->m_fSumOfX += pfArray[i];
    pStatData->m_fSumOfXSqr += pfArray[i] * pfArray[i];
  }
}

void calcBasicStat(BasicStat* pStatData)
{
  if (pStatData->m_nNumData < 2)
    return;
  // calculate mean value
  pStatData->m_fMean = pStatData->m_fSumOfX / pStatData->m_nNumData;
  // calculate standard deviation
  pStatData->m_fSdev =
       sqrt((pStatData->m_fSumOfXSqr -
             pStatData->m_fSumOfX * pStatData->m_fSumOfX /
             pStatData->m_nNumData) /
             (pStatData->m_nNumData - 1));

}

