// A C++ program that illustrates
// pointers to dynamic variables and class instances

#include <iostream.h>
#include <iomanip.h>

const int MAX_ELEMS = 10;

class myArray
{
  public:
    myArray(int nMaxElems = MAX_ELEMS, int fInitVal = 0);
    ~myArray()
      { delete [] m_pnArray; }
    int& operator[](int nIndex)
      { return m_pnArray[nIndex]; }
    void show(const char* pszMsg = "",
              const int nNumElems = MAX_ELEMS,
              const int bOneLine = 1);
    void ShellSort(int nNumElems = MAX_ELEMS);

  protected:
    int* m_pnArray;
    int m_nMaxElems;
};

myArray::myArray(int nMaxElems, int fInitVal)
{
  m_nMaxElems = (nMaxElems > 1) ? nMaxElems : 1;
  m_pnArray = new int[m_nMaxElems];
  for (int i = 0; i < m_nMaxElems; i++)
    m_pnArray[i] = fInitVal;
}

void myArray::show(const char* pszMsg,
                   const int nNumElems,
                   const int bOneLine)
{
  cout << pszMsg << endl;
  if (bOneLine) {
    for (int i = 0; i < nNumElems; i++)
      cout << m_pnArray[i] << ' ';
    cout << endl;
  }
  else {
    for (int i = 0; i < nNumElems; i++)
      cout << m_pnArray[i] << endl;
    cout << endl;
  }
}

void myArray::ShellSort(int nNumElems)
{
  int nOffset = nNumElems;
  int bSorted;

  if (nNumElems < 2 ||
      nNumElems > m_nMaxElems)
    return;

  while (nOffset > 1) {
    nOffset = (nOffset - 1) / 3;
    nOffset = (nOffset < 1) ? 1 : nOffset;
    // arrange elements that are nOffset apart
    do {
      bSorted = 1; // set sorted flag
      // compare elements
      for (int i = 0, j = nOffset;
           i < (nNumElems - nOffset);
           i++, j++) {
        if (m_pnArray[i] > m_pnArray[j]) {
          // swap elements
          int nSwap = m_pnArray[i];
          m_pnArray[i] = m_pnArray[j];
          m_pnArray[j] = nSwap;
          bSorted = 0; // clear sorted flag
        }
      }
    } while (!bSorted);
  }
}

main()
{
  int nArr[MAX_ELEMS] = { 23, 45, 89, 74, 51,
                          18, 87, 63, 36, 38 };
  int* pnArray = new int[MAX_ELEMS];

  // copy array elements
  for (int i = 0; i < MAX_ELEMS; i++)
    pnArray[i] = nArr[i];

  cout << "Unsorted array:\n";
  for (i = 0; i < MAX_ELEMS; i++)
    cout << pnArray[i] << ' ';
  cout << endl;

  for (i = 0; i < (MAX_ELEMS - 1); i++)
    for (int j = i+1; j < MAX_ELEMS; j++)
       if (pnArray[i] < pnArray[j]) {
         // allocate dynamic swap buffer
         int* pnSwap = new int;
         *pnSwap = pnArray[i];
         pnArray[i] = pnArray[j];
         pnArray[j] = *pnSwap;
         // deallocate dynamic swap buffer
         delete pnSwap;
       }

  cout << "Sorted array:\n";
  for (i = 0; i < MAX_ELEMS; i++)
    cout << pnArray[i] << ' ';
  cout << endl << endl;

  delete [] pnArray;

  myArray* pArray =  new myArray;

  // copy array elements
  for (i = 0; i < MAX_ELEMS; i++)
    pArray->operator[](i) = nArr[i];

  pArray->show("Unsorted array is:");
  pArray->ShellSort();
  pArray->show("Sorted array is:");

  delete pArray;

  return 0;
}

