// A C++ program illustrates passing
// structures as function parameters

#include <iostream.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

const double BAD_RESULT = -1.0e+30;

inline double sqr(double x)
{ return x * x; }

// declare the structure for basic statistical data
struct BasicStat {
  int m_nNumData;
  double m_fSumX;
  double m_fSumXSqr;
};

// declare prototypes
BasicStat initBasicStat();
BasicStat addData(double fArray[], int nNumElems,
                  BasicStat BasStat);
double getMean(BasicStat BasStat);
double getSdev(BasicStat BasStat);

main()
{
  const int MAX_SETS = 3;
  const int MAX = 100;
  const int MIN_NUM = 1;
  const int MAX_NUM = 1000;
  double fData[MAX];
  double fMean, fSdev;
  int i;
  // declare a structure
  BasicStat StatData;

  // reseed random-number generator
  srand((unsigned)time(NULL));

  // initialize the structure variable StatData
  StatData = initBasicStat();

  for (int nDataSet = 0; nDataSet < MAX_SETS; nDataSet++) {
    // generate random data
    for (i = 0; i < MAX; i++)
      fData[i] = double(rand() % MAX_NUM + MIN_NUM);
    // update statistical summations
    StatData = addData(fData, MAX, StatData);
  }

  // calculate mean value
  fMean = getMean(StatData);
  // calculate standard deviation
  fSdev = getSdev(StatData);

  cout << "Number of observations = "
       << StatData.m_nNumData << "\n"
       << "Mean = " << fMean << "\n"
       << "Standard deviation = "
       << fSdev;

  return 0;
}

BasicStat initBasicStat()
{
  BasicStat BasStat = { 0, 0.0, 0.0 };
  return BasStat;
}

BasicStat addData(double fArray[], int nNumElems,
                  BasicStat BasStat)
{
  BasStat.m_nNumData += nNumElems;
  for (int i = 0; i < nNumElems; i++) {
    BasStat.m_fSumX += fArray[i];
    BasStat.m_fSumXSqr += sqr(fArray[i]);
  }
  return BasStat;
}

double getMean(BasicStat BasStat)
{
  if (BasStat.m_nNumData > 0)
    return BasStat.m_fSumX / BasStat.m_nNumData;
   else
    return BAD_RESULT;
}

double getSdev(BasicStat BasStat)
{
  if (BasStat.m_nNumData > 0)
   return sqrt((BasStat.m_fSumXSqr -
                sqr(BasStat.m_fSumX) / BasStat.m_nNumData) /
                (BasStat.m_nNumData - 1));
  else
    return BAD_RESULT;
}

