// A C++ program that illustrates
// the template parameters

#include <iostream.h>
#include <iomanip.h>

enum Logical { FALSE, TRUE };

template <class T, int nSize>
class myArray
{
  public:
    myArray(T InitVal);
    T& operator[](int nIndex)
      { return m_Array[nIndex]; }
    void show(const int nNumElems,
              const char* pszMsg = "",
              const Logical bOneLine = TRUE);
    void CombSort(int nNumElems);

  protected:
    T m_Array[nSize];
    int m_nSize;
};

template <class T, int nSize>
myArray<T, nSize>::myArray(T InitVal)
{
  m_nSize = nSize;
  for (int i = 0; i < m_nSize; i++)
    m_Array[i] = InitVal;
}

template <class T, int nSize>
void myArray<T, nSize>::show(const int nNumElems,
                             const char* pszMsg,
                             const Logical bOneLine)
{
  cout << pszMsg << endl;
  if (bOneLine) {
    for (int i = 0; i < nNumElems; i++)
      cout << m_Array[i] << ' ';
    cout << endl;
  }
  else {
    for (int i = 0; i < nNumElems; i++)
      cout << m_Array[i] << endl;
    cout << endl;
  }
}

template <class T, int nSize>
void myArray<T, nSize>::CombSort(int nNumElems)
{
  int nOffset = nNumElems;
  Logical bSorted;

  // check argument of parameter nNumElems
  if (nNumElems < 2)
    return;

  do {
    nOffset = (nOffset * 8) / 11;
    nOffset = (nOffset < 1) ? 1 : nOffset;
    bSorted = TRUE; // set sorted flag
    // compare elements
    for (int i = 0, j = nOffset;
         i < (nNumElems - nOffset);
         i++, j++) {
      if (m_Array[i] > m_Array[j]) {
        // swap elements
        T nSwap = m_Array[i];
        m_Array[i] = m_Array[j];
        m_Array[j] = nSwap;
        bSorted = FALSE; // clear sorted flag
      }
    }
  } while (!bSorted || nOffset != 1);
}

main()
{
  const int MAX_ELEMS = 10;
  int nArr[MAX_ELEMS] = { 23, 45, 89, 74, 51,
                          18, 87, 63, 36, 38 };
  int cArr[MAX_ELEMS] = { 'a', 'W', 'Q', 'y', 'H',
                          'L', 'J', 'M', 'd', 'D' };

  myArray<int, MAX_ELEMS> IntegerArray(0);
  myArray<char, MAX_ELEMS> CharArray(' ');

  // assign integers to elements of array IntegerArray
  for (int i = 0; i < MAX_ELEMS; i++)
    IntegerArray[i] = nArr[i];
  // assign characters to elements of array CharArray
  for (i = 0; i < MAX_ELEMS; i++)
    CharArray[i] = cArr[i];

  // test array IntegerArray
  IntegerArray.show(MAX_ELEMS, "Unsorted array is:");
  IntegerArray.CombSort(MAX_ELEMS);
  IntegerArray.show(MAX_ELEMS, "Sorted array is:");
  cout << "\n\n";

  // test array CharArray
  CharArray.show(MAX_ELEMS, "Unsorted array is:");
  CharArray.CombSort(MAX_ELEMS);
  CharArray.show(MAX_ELEMS, "Sorted array is:");
  cout << "\n\n";

  return 0;
}

