// A C++ program that illustrates
// the template parameters

#include <iostream.h>
#include <iomanip.h>

enum Logical { FALSE, TRUE };

template <class T, class IDX, int nSize>
class myArray
{
  public:
    myArray(T InitVal);
    T& operator[](IDX nIndex)
      { return m_Array[nIndex]; }
    void show(const int nNumElems,
              const char* pszMsg = "",
              const Logical bOneLine = TRUE);
    void CombSort(int nNumElems);

  protected:
    T m_Array[nSize];
    int m_nSize;
};

template <class T, class IDX, int nSize>
myArray<T, IDX, nSize>::myArray(T InitVal)
{
  m_nSize = nSize;
  for (IDX i = 0; i < m_nSize; i++)
    m_Array[i] = InitVal;
}

template <class T, class IDX, int nSize>
void myArray<T, IDX, nSize>::show(const int nNumElems,
                             const char* pszMsg,
                             const Logical bOneLine)
{
  cout << pszMsg << endl;
  if (bOneLine) {
    for (IDX i = 0; i < nNumElems; i++)
      cout << m_Array[i] << ' ';
    cout << endl;
  }
  else {
    for (IDX i = 0; i < nNumElems; i++)
      cout << m_Array[i] << endl;
    cout << endl;
  }
}

template <class T, class IDX, int nSize>
void myArray<T, IDX, nSize>::CombSort(int nNumElems)
{
  int nOffset = nNumElems;
  Logical bSorted;

  // check argument of parameter nNumElems
  if (nNumElems < 2)
    return;

  do {
    nOffset = (nOffset * 8) / 11;
    nOffset = (nOffset < 1) ? 1 : nOffset;
    bSorted = TRUE; // set sorted flag
    // compare elements
    for (IDX i = 0, j = nOffset;
         i < (nNumElems - nOffset);
         i++, j++) {
      if (m_Array[i] > m_Array[j]) {
        // swap elements
        T nSwap = m_Array[i];
        m_Array[i] = m_Array[j];
        m_Array[j] = nSwap;
        bSorted = FALSE; // clear sorted flag
      }
    }
  } while (!bSorted || nOffset != 1);
}

main()
{
  const int MAX_ELEMS = 10;
  int nArr[MAX_ELEMS] = { 23, 45, 89, 74, 51,
                          18, 87, 63, 36, 38 };

  myArray<int, int, MAX_ELEMS> IntegerArray(0);
  myArray<int, char, MAX_ELEMS> CharArray(' ');

  // assign values to elements of array IntegerArray
  for (int i = 0; i < MAX_ELEMS; i++)
    IntegerArray[i] = nArr[i];
  // assign values to elements of array CharArray
  for (char c = 0; c < MAX_ELEMS; c++)
    CharArray[c] = nArr[int(c)];

  // test array IntegerArray
  IntegerArray.show(MAX_ELEMS, "Unsorted array is:");
  IntegerArray.CombSort(MAX_ELEMS);
  IntegerArray.show(MAX_ELEMS, "Sorted array is:");
  cout << "\n\n";

  // test array CharArray
  CharArray.show(MAX_ELEMS, "Unsorted array is:");
  CharArray.CombSort(MAX_ELEMS);
  CharArray.show(MAX_ELEMS, "Sorted array is:");
  cout << "\n\n";

  return 0;
}

